--[[
	Copyright (C) 2007 Nymbia

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
]]
local Manufac = Manufac
local L = AceLibrary("AceLocale-2.2"):new("ManufacSnoop")
local LM = AceLibrary("AceLocale-2.2"):new("Manufac")
ManufacSnoop = Manufac:NewModule("Snoop","AceHook-2.1")
local dewdrop = AceLibrary("Dewdrop-2.0")
local BS = AceLibrary("Babble-Spell-2.2")
local PT = AceLibrary("PeriodicTable-3.0")
local function colors(diff)
	local percent = diff/375
	if percent > 1 then
		percent = 1
	end
	if percent >= 0.6666 then
		--RANGE: FF8844 (TOP) to FFFF00 (BOTTOM)
		--  1.000,0.531,0.265 to 1.000,1.000,0.000
		percent = (percent - 0.6666) * 3
		return ("|cff%02x%02x%02x"):format(255,math.floor((1-percent*0.469)*255),math.floor((percent*0.265)*255))
	elseif percent >= 0.3333 then
		--RANGE: FFFF00 (TOP) to 44CC44 (BOTTOM)
		--  1.000,1.000,0.000 to 0.265,0.786,0.265
		percent = (percent - 0.3333) * 3
		return ("|cff%02x%02x%02x"):format((0.265+(percent*0.735))*255,(0.786+(percent*0.214))*255,(0.265-(percent*0.265))*255)
	else
		--RANGE: 44CC44 (TOP) to AAAAAA (BOTTOM)
		--  0.265,0.786,0.265 to 0.664,0.664,0.664
		percent = percent * 3
		return ("|cff%02x%02x%02x"):format((0.664-(percent*0.399))*255,(0.664+(percent*0.122))*255,(0.664-(percent*0.399))*255)
	end
end
function ManufacSnoop:OnInitialize()
	CreateFrame("GameTooltip","ManufacSnoopTooltip",UIParent,"GameTooltipTemplate")
	self.db = Manufac:AcquireDBNamespace("Snoop")
	Manufac:RegisterDefaults("Snoop","profile",{
		showprofessions = true,
		showmissing = true,
	})
	Manufac.options.args.modules.args.Snoop = {
		type = 'group',
		name = "Snoop",
		desc = "Snoop",
		args = {
			toggle = {
				name = LM["Toggle"],
				type = 'toggle',
				desc = LM["Toggle"],
				get = function()
					return Manufac:IsModuleActive("Snoop")
				end,
				set = function(v)
					Manufac:ToggleModuleActive("Snoop",v)
				end,
				order = 101,
			},
			showprofessions = {
				name = L["Show Professions"],
				type = 'toggle',
				desc = L["Show Professions"],
				get = function()
					return self.db.profile.showprofessions
				end,
				set = function(v)
					self.db.profile.showprofessions = v
				end,
			},
			showmissing = {
				name = L["Show Missing"],
				type = 'toggle',
				desc = L["Show Missing"],
				get = function()
					return self.db.profile.showmissing
				end,
				set = function(v)
					self.db.profile.showmissing = v
				end,
			},
			--[[
			cacheall = {
				name = L["Cache All"],
				type = 'execute',
				desc = L["Cache All"],
				func = function()
					for id, reagents in PT:IterateSet("TradeskillResultMats.Forward") do
						ManufacSnoop:GetItemData(id)
						for reagentid in reagents:gmatch("(%d+)x%d+") do
							ManufacSnoop:GetItemData(id)
						end
					end
				end,
				order = -5
			},
			]]
		},
	}
end
local function itemsort(a,b)
	if Manufac.db.profile.sorttype == LM["Alpha"] then
		if tonumber(a) and not tonumber(b) then
			return false
		elseif tonumber(b) and not tonumber(a) then
			return true
		else
			return a < b
		end
	else
		local adiff = ("%03d"):format((PT:ItemInSet(a,"Tradeskill.Crafted")))
		local bdiff = ("%03d"):format((PT:ItemInSet(b,"Tradeskill.Crafted")))
		if not adiff or not bdiff or adiff == bdiff then
			if tonumber(a) and not tonumber(b) then
				return false
			elseif tonumber(b) and not tonumber(a) then
				return true
			else
				return a < b
			end
		else
			return adiff > bdiff
		end
	end
end
local temp = {}
local temp2 = {}
function ManufacSnoop:Menu(level,arg1,arg2,arg3,arg4,arg5)
	if level == 1 then
		for k,_ in pairs(temp) do
			temp[k] = nil
		end
		for k,_ in pairs(temp2) do
			temp2[k] = nil
		end
		-- This is a profession the player doesn't have, make the group entries
		local prof = arg1:match("|([^%|]+)$")
		dewdrop:AddLine(
			'text', prof,
			'isTitle', true
		)
		dewdrop:AddLine()
		for item,value,set in PT:IterateSet("Tradeskill.Crafted."..BS:GetReverseTranslation(prof)) do
			temp[Manufac:LinkToSet(prof, item)] = true
		end
		for k,_ in pairs(temp) do
			table.insert(temp2,k)
		end
		table.sort(temp2)
		for _,v in ipairs(temp2) do
			dewdrop:AddLine(
				'text', v,
				'value', "|SNOOP2|"..v,
				'hasArrow', true
			)
		end
	elseif level == 2 then
		if arg2:sub(1,6) == "|SNOOP" then
			-- The player doesnt have this profession, which means we're pointed at a certain group.
			-- Make lines for recipes in that group.
			for k,_ in pairs(temp) do
				temp[k] = nil
			end
			local thisgroup = arg1:sub(9)
			local prof = arg2:match("|([^%|]+)$")
			for item,value,set in PT:IterateSet("Tradeskill.Crafted."..BS:GetReverseTranslation(prof)) do
				local group = Manufac:LinkToSet(prof, item)
				if group == thisgroup then
					local link,_
					if prof == BS["Enchanting"] then
						--ugly hax to get the name of the enchant.
						link = "|cffffffff|Henchant:"..(-1 * item).."|h[lol who knows]|h|r"
						ManufacSnoopTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
						ManufacSnoopTooltip:SetHyperlink(link)
						ManufacSnoopTooltip:Show()
						local name = ManufacSnoopTooltipTextLeft1:GetText()
						ManufacSnoopTooltip:Hide()
						if name then
							link = "|cffffffff|Henchant:"..(-1*item).."|h["..name.."]|h|r"
						end
					else
						_,link = GetItemInfo(item)
					end
					if not link then
						link = item
					end
					table.insert(temp,link)
				end
			end
			table.sort(temp,itemsort)
			for _,v in ipairs(temp) do
				self:AddItemLine(v, BS:GetReverseTranslation(prof))
			end
		else
			-- The player has this profession, which means we need to print out group entries.
			
		end
	elseif level == 3 then
		if arg3:sub(1,6) == "|SNOOP" then
			-- The player doesnt have this profession, which means we're pointed at a certain item.
			local link = arg1:match("^|[^%|]+|(.+)$")
			local prof = arg3:match("|([^%|]+)$")
			local englishprof = BS:GetReverseTranslation(prof)
			local difficulty = PT:ItemInSet(link,"Tradeskill.Crafted."..englishprof)
			local color = colors(difficulty)
			local reagentstring = PT:ItemInSet(link, "TradeskillResultMats.Forward."..englishprof)
			local count = 1000
			local countwbank = 1000
			local name = link:match("|h%[([^%]]+)%]|h")
			local texture
			if Manufac.db.profile.icons then
				if prof == "Enchanting" then
					texture = "Interface\\Icons\\Spell_Holy_GreaterHeal"
				else
					texture = select(10,GetItemInfo(link))
				end
			end
			for reagentid, num in reagentstring:gmatch("(%d+)x(%d+)") do
				reagentid, num = tonumber(reagentid), tonumber(num)
				local max = math.floor(GetItemCount(reagentid) / num)
				if max < count then
					count = max
				end
				local bankmax = math.floor(GetItemCount(reagentid, true) / num)
				if bankmax < countwbank then
					countwbank = bankmax
				end
			end
			local countstring
			if countwbank == 0 then
				countstring = ""
			else
				countstring = " ["..count.."/"..countwbank.."]"
			end
			if tonumber(link) then
				-- Uncached item
				dewdrop:AddLine(
					'text', color.."["..link.."] ("..difficulty..")"..countstring
				)
				dewdrop:AddLine(
					'text', "|cffff2222"..L["Click to Cache"],
					'func', self.GetItemData,
					'arg1', self, 'arg2', link
				)
			else
				-- cached item, yay!
				dewdrop:AddLine(
					'text', color.."["..name.."] ("..difficulty..")"..countstring,
					'func', Manufac.PrintSingleLink,
					'arg1', Manufac, 'arg2', link,
					'icon', texture,
					'tooltipFunc', Manufac.SetTooltip,
					'tooltipArg1', GameTooltip, 'tooltipArg2', link
				)
				if Manufac.db.profile.money then
					local sell, buy, buyout = Manufac:GetPriceData(link)
					if buyout and buyout > 0 then
						dewdrop:AddLine(
							'text', ("Market Value: "..Manufac:GetMoneyText(buyout)),
							'isTitle', true
						)
					end
					if sell and sell > 0 then
						dewdrop:AddLine(
							'text', ("Vendor Value: "..Manufac:GetMoneyText(sell)),
							'isTitle', true
						)
					end
				end
			end
			dewdrop:AddLine()
			local all = true
			for reagentid, num in reagentstring:gmatch("(%d+)x(%d+)") do
				if not self:AddReagentLine(reagentid, num) then
					all = false
				end
			end
			if not tonumber(link) then
				dewdrop:AddLine()
				dewdrop:AddLine(
					'text', ("|cFFFFFF00"..LM["LinkResult"].."|r"),
					'func', Manufac.PrintSingleLink,
					'arg1', Manufac,
					'arg2', link
				)
				if all then
					dewdrop:AddLine(
						'text', ("|cFFFFFF00"..LM["LinkRecipe!"].."|r"),
						'func', self.PrintMultiLink,
						'arg1', self,
						'arg2', link,
						'arg3', reagentstring
					)
				end
			end
		else
			-- The player has this profession, print group member item's lines.
			
		end
	elseif level >= 4 then
		-- We're pointed at an item.
		
	end
end
function ManufacSnoop:AddItemLine(link, profession)
	-- link is either an actual link (if its cached) or an itemid or a negative spell id (if it isnt)
	local difficulty = PT:ItemInSet(link, "Tradeskill.Crafted."..profession)
	local reagentstring = PT:ItemInSet(link, "TradeskillResultMats.Forward."..profession)
	local color = colors(difficulty)
	local texture
	if Manufac.db.profile.icons then
		if profession == BS["Enchanting"] then
			texture = "Interface\\Icons\\Spell_Holy_GreaterHeal"
		else
			texture = select(10,GetItemInfo(link))
		end
	end
	if tonumber(link) then
		-- this item's not cached, no fancy stuff :(
		dewdrop:AddLine(
			'text', color.."["..link.."]",
			'value', "|SNOOP2|"..link,
			'hasArrow', true,
			'func', self.GetItemData,
			'arg1', self, 'arg2', link
		)
		return
	end
	local count = 1000
	local countwbank = 1000
	for reagentid, num in reagentstring:gmatch("(%d+)x(%d+)") do
		reagentid, num = tonumber(reagentid), tonumber(num)
		local max = math.floor(GetItemCount(reagentid) / num)
		if max < count then
			count = max
		end
		local bankmax = math.floor(GetItemCount(reagentid, true) / num)
		if bankmax < countwbank then
			countwbank = bankmax
		end
	end
	local name = link:match("%|h%[([^%]]+)%]%|h")
	if countwbank > 0 then
		if Manufac.db.profile.l2tooltips then
			dewdrop:AddLine(
				'text', color..name.."|r ["..count.."/"..countwbank.."]",
				'value', "|SNOOP2|"..link,
				'hasArrow', true,
				'tooltipFunc', Manufac.SetTooltip,
				'tooltipArg1', GameTooltip, 'tooltipArg2', link,
				'icon', texture
			)
		else
			dewdrop:AddLine(
				'text', color..name.."|r ["..count.."/"..countwbank.."]",
				'value', "|SNOOP2|"..link,
				'hasArrow', true,
				'tooltipFunc', nil,
				'icon', texture
			)
		end
	else
		if Manufac.db.profile.l2tooltips then
			dewdrop:AddLine(
				'text', color..name,
				'value', "|SNOOP2|"..link,
				'hasArrow', true,
				'tooltipFunc', Manufac.SetTooltip,
				'tooltipArg1', GameTooltip, 'tooltipArg2', link,
				'icon', texture
			)
		else
			dewdrop:AddLine(
				'text', color..name,
				'value', "|SNOOP2|"..link,
				'hasArrow', true,
				'tooltipFunc', nil,
				'icon', texture
			)
		end
	end
end
function ManufacSnoop:AddReagentLine(id, num)
	id, num = tonumber(id), tonumber(num)
	local name,link,_,_,_,_,_,_,_,texture = GetItemInfo(id)
	if not Manufac.db.profile.icons then
		texture = nil
	end
	if not link then
		dewdrop:AddLine(
			'text', "|cFF6666FF["..id.."]",
			'func', self.GetItemData,
			'arg1', self, 'arg2', id
		)
		return false
	end
	local count = GetItemCount(id)
	local bankcount = GetItemCount(id,true)
	local bankstr
	local price
	if Manufac.db.profile.money then
		local sell, buy, buyout = Manufac:GetPriceData(link)
		if buy and buy > 0 and (((not buyout) or buyout == 0) or (not Auctioneer)) then
			price = buy*info[i]["needed"]
			totalprice = price + totalprice
			price = Manufac:GetMoneyText(price)
		elseif buyout and buyout > 0 then
			price = buyout*info[i]["needed"]
			totalprice = price + totalprice
			price = Manufac:GetMoneyText(price)
		else
			price = LM["Unknown"]
		end
	else
		price = ""
	end
	if bankcount > count then
		bankstr = LM["(bank: "]..(bankcount-count)..")"
	else
		bankstr = ""
	end
	dewdrop:AddLine(
		'text', ("|cFF6666FF"..name.."|r ["..count.."/"..num.."] "..bankstr.." "..price),
		'func', Manufac.PrintSingleLink,
		'arg1',Manufac,
		'arg2', link,
		'tooltipFunc', Manufac.SetTooltip,
		'tooltipArg1', GameTooltip, 'tooltipArg2', link,
		'icon', texture
	)
	return true
end
function ManufacSnoop:PrintMultiLink(link, reagentstring)
	local str = link..LM[" requires: "]
	for reagentid, num in reagentstring:gmatch("(%d+)x(%d+)") do
		local name = GetItemInfo(tonumber(reagentid))
		str = str..num.."x "..name..", "
	end
	str = str:sub(1,-3)
	if ( ChatFrameEditBox:IsVisible() ) then
		ChatFrameEditBox:Insert(str)
	else
		ChatFrame_OpenChat(str)
	end
end
function ManufacSnoop:GetItemData(id)
	ManufacSnoopTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
	ManufacSnoopTooltip:SetHyperlink("item:" .. id)
	ManufacSnoopTooltip:Hide()
end